#!/usr/bin/env python2.7

import argparse
from cert_tree import Cert, extract_certs_as_strings, create_certs, create_cert, construct_tree
import sys
import os

def get_cert_list_by_tree(roots, arr):
    for root in roots:
        arr.append(root)
        get_cert_list_by_tree(root.children, arr)

def list_files_recursive(path):
    files = []
    # r = root, d = directories, f = files
    for r, d, f in os.walk(path):
        for file in f:
            files.append(os.path.join(r, file))

    return [file for file in files]

def get_ca_list(certs):
    result = []
    for certStr in certs:
        cert = create_cert(extract_certs_as_strings(certStr)[0], 1)
        result.append(cert)

    return result

def main():
    parser = argparse.ArgumentParser(description='View tree of certificates from pem file')
    parser.add_argument('cert_file', help='the cert file in pem format')
    parser.add_argument('ca_path', help="position of root ca cert")
    parser.add_argument('output', help="position of ca bundle cert")

    args = parser.parse_args()

    certs = extract_certs_as_strings(args.cert_file)
    if not certs:
        raise Exception('No certs found in the pem file')

    certTree = construct_tree(create_certs(certs))

    certList = []
    get_cert_list_by_tree(certTree, certList)

    result = []
    isFinished = False
    caList = get_ca_list(list_files_recursive(args.ca_path))
    for cert in reversed(certList):
        result.append(cert)
        for ca in caList:
            if ca.subject == cert.issuer:
                # reset first trusted root certificate
                result.append(ca)
                isFinished = True
                break
        if isFinished:
            break

    f = open(args.output, "w")
    for ca in result:
        f.write(ca.content)
    f.close()

    # update intermediate certficate without root ca
    f = open(args.cert_file, "w")
    for cert in result[:-1]:
        f.write(cert.content)
    f.close()

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        print (e)
        sys.exit(-1)
    except:
        print ('unknown error')
        sys.exit(-1)

    sys.exit(0)
